"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require("mkdirp");

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _glob = require("glob");

var _glob2 = _interopRequireDefault(_glob);

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _locales = require("./utils/locales");

var locales = _interopRequireWildcard(_locales);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sourceFilename = "{locale}/messages.json";
var compiledFilename = "{locale}/messages.js";

exports.default = function (config) {
  return {
    formatFilename(pattern, locale) {
      return pattern.replace("{locale}", locale);
    },

    write(locale, messages) {
      var filename = _path2.default.join(config.localeDir, this.formatFilename(sourceFilename, locale));

      var created = !_fs2.default.existsSync(filename);
      _fs2.default.writeFileSync(filename, (0, _stringify2.default)(messages, null, 2));
      return [created, filename];
    },

    read(locale) {
      var filename = _path2.default.join(config.localeDir, this.formatFilename(sourceFilename, locale));

      if (!_fs2.default.existsSync(filename)) return null;

      var raw = _fs2.default.readFileSync(filename).toString();

      try {
        return JSON.parse(raw);
      } catch (e) {
        return null;
      }
    },

    merge(nextCatalog) {
      var _this = this;

      var nextKeys = _ramda2.default.keys(nextCatalog);

      return _ramda2.default.mergeAll(this.getLocales().map(function (locale) {
        var prevCatalog = _this.read(locale);
        if (!prevCatalog) return nextCatalog;

        var prevKeys = _ramda2.default.keys(prevCatalog);

        var newKeys = _ramda2.default.difference(nextKeys, prevKeys);
        var mergeKeys = _ramda2.default.intersection(nextKeys, prevKeys);
        var obsoleteKeys = _ramda2.default.difference(prevKeys, nextKeys);

        var newMessages = _ramda2.default.map(function (message) {
          return (0, _extends3.default)({
            translation: ""
          }, message);
        }, _ramda2.default.pick(newKeys, nextCatalog));

        var mergedMessages = mergeKeys.map(function (key) {
          return {
            [key]: (0, _extends3.default)({
              translation: prevCatalog[key].translation
            }, _ramda2.default.omit(["obsolete, translation"], nextCatalog[key]))
          };
        });

        var obsoleteMessages = obsoleteKeys.map(function (key) {
          return {
            [key]: (0, _extends3.default)({}, prevCatalog[key], {
              obsolete: true
            })
          };
        });

        var newCatalog = _ramda2.default.mergeAll([newMessages].concat((0, _toConsumableArray3.default)(mergedMessages), (0, _toConsumableArray3.default)(obsoleteMessages)));
        return { [locale]: newCatalog };
      }));
    },

    getTranslation(catalogs, locale, key, _ref) {
      var fallbackLocale = _ref.fallbackLocale,
          sourceLocale = _ref.sourceLocale;

      var getTranslation = function getTranslation(locale) {
        return catalogs[locale][key].translation;
      };

      return getTranslation(locale) || fallbackLocale && getTranslation(fallbackLocale) || catalogs[locale][key].defaults || sourceLocale && sourceLocale === locale && key || sourceLocale && fallbackLocale && sourceLocale === fallbackLocale && key || undefined;
    },

    writeCompiled(locale, content) {
      var filename = _path2.default.join(config.localeDir, this.formatFilename(compiledFilename, locale));

      _fs2.default.writeFileSync(filename, content);
      return filename;
    },

    getLocale(filename) {
      var filenameRe = new RegExp(this.formatFilename(sourceFilename, locales.localeRe.source));
      var match = filenameRe.exec(filename);
      if (!match) return null;

      return match[1];
    },

    getLocales() {
      var _this2 = this;

      var pattern = _path2.default.join(config.localeDir, this.formatFilename(sourceFilename, "*"));

      return _glob2.default.sync(pattern).map(function (filename) {
        return _this2.getLocale(filename);
      }).filter(Boolean);
    },

    addLocale(locale) {
      if (!locales.isValid(locale)) {
        return [false, null];
      }

      var filename = _path2.default.join(config.localeDir, this.formatFilename(sourceFilename, locale));

      if (!_fs2.default.existsSync(filename)) {
        var dirname = _path2.default.dirname(filename);

        _mkdirp2.default.sync(dirname);
        this.write(locale, {});

        return [true, filename];
      }

      return [false, filename];
    }
  };
};