"use strict";

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _commander = require("commander");

var _commander2 = _interopRequireDefault(_commander);

var _makePlural = require("make-plural");

var _makePlural2 = _interopRequireDefault(_makePlural);

var _linguiConf = require("lingui-conf");

var _linguiConf2 = _interopRequireDefault(_linguiConf);

var _compile = require("./api/compile");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function command(config, format, options) {
  var locales = format.getLocales();

  if (!locales.length) {
    console.log("No locales defined!\n");
    console.log(`(use "${_chalk2.default.yellow("lingui add-locale <language>")}" to add one)`);
    return false;
  }

  var catalogs = _ramda2.default.mergeAll(locales.map(function (locale) {
    return { [locale]: format.read(locale) };
  }));

  var noMessages = _ramda2.default.compose(_ramda2.default.all(_ramda2.default.equals(true)), _ramda2.default.values, _ramda2.default.map(_ramda2.default.isEmpty));
  if (noMessages(catalogs)) {
    console.log("Nothing to compile, message catalogs are empty!\n");
    console.log(`(use "${_chalk2.default.yellow("lingui extract")}" to extract messages from source files)`);
    return false;
  }

  console.log("Compiling message catalogs…");

  return locales.map(function (locale) {
    var _locale$split = locale.split("_"),
        _locale$split2 = (0, _slicedToArray3.default)(_locale$split, 1),
        language = _locale$split2[0];

    if (!_makePlural2.default[language]) {
      console.log(_chalk2.default.red(`Error: Invalid locale ${_chalk2.default.bold(locale)} (missing plural rules)!`));
      console.log();
      return false;
    }

    var messages = _ramda2.default.mergeAll((0, _keys2.default)(catalogs[locale]).map(function (key) {
      return {
        [key]: format.getTranslation(catalogs, locale, key, {
          fallbackLocale: config.fallbackLocale,
          sourceLocale: config.sourceLocale
        })
      };
    }));

    if (!options.allowEmpty && config.sourceLocale !== locale) {
      var missing = _ramda2.default.keys(messages).filter(function (key) {
        return messages[key] === undefined;
      });

      if (missing.length) {
        console.log(_chalk2.default.red(`Error: Failed to compile catalog for locale ${_chalk2.default.bold(locale)}!`));

        if (options.verbose) {
          console.log(_chalk2.default.red("Missing translations:"));
          missing.forEach(function (msgId) {
            return console.log(msgId);
          });
        } else {
          console.log(_chalk2.default.red(`Missing ${missing.length} translation(s)`));
        }
        console.log();
        return false;
      }
    }

    var compiledCatalog = (0, _compile.createCompiledCatalog)(locale, messages);
    var compiledPath = format.writeCompiled(locale, compiledCatalog);

    options.verbose && console.log(_chalk2.default.green(`${locale} ⇒ ${compiledPath}`));
    return compiledPath;
  });
}

if (require.main === module) {
  _commander2.default.description("Add compile message catalogs and add language data (plurals) to compiled bundle.").option("--strict", "Disable defaults for missing translations").option("--verbose", "Verbose output").option("--format <format>", "Format of message catalog").on("--help", function () {
    console.log("\n  Examples:\n");
    console.log("    # Compile translations and use defaults or message IDs for missing translations");
    console.log("    $ lingui compile");
    console.log("");
    console.log("    # Compile translations but fail when there're missing");
    console.log("    # translations (don't replace missing translations with");
    console.log("    # default messages or message IDs)");
    console.log("    $ lingui compile --strict");
  }).parse(process.argv);

  var config = (0, _linguiConf2.default)();
  var formatName = _commander2.default.format || config.format;
  var format = require(`./api/formats/${formatName}`).default(config);

  var results = command(config, format, {
    verbose: _commander2.default.verbose || false,
    allowEmpty: !_commander2.default.strict
  });

  if (!results || results.some(function (res) {
    return !res;
  })) {
    process.exit(1);
  }

  console.log("Done!");
}